#ifndef IKD_OX_BASIC_OXLAYOUT_H
#define IKD_OX_BASIC_OXLAYOUT_H

#include "oxcontainer.h"
#include "oxfloat.h"

namespace OX {
	namespace Layout {
		///////////////////////////
		// ̈
		/////
		class Region {
		protected:
			Vector<Float2> vertices;	// _W
			Float2 pivot;				// ]s{bg

		public:
			Region() : pivot(0.0f, 0.0f) {}
			void setPivot(const Float2 &pos) {pivot = pos;}
			const Float2 &getPivot() const {return pivot;}
			Vector<Float2> &getVertices() {return vertices;}
			virtual bool isContain(const Float2 &pos) = 0;
		};


		///////////////////////////
		// `̈
		/////
		class RectRegion : public Region {
			float w, h, l, t;

		public:
			RectRegion();
			RectRegion(float w, float h, float l, float t);
			virtual ~RectRegion();
			void create(float w, float h, float l, float t);
			virtual bool isContain(const Float2 &pos);
		};


		///////////////////////////
		// p[^
		/////
		struct Param {
			Float2 trans;		// ړ
			float rot;			// ]ʁiDegreej
			unsigned color;		// F
		};


		///////////////////////////
		// }EX
		/////
		struct Mouse {
			enum CLICK {L, R, M};
			struct Data {
				union {
					struct{Float2 pos;};
					struct {float x, y;};
				};
				char flag;		// tO
				Data() : x(), y(), flag() {}
			};

			Data pre, cur;

			// XV
			void update(const Mouse::Data &data) {
				pre = cur;
				cur = data;
			}

			// ʒu擾
			const Float2 &getPos() const {
				return cur.pos;
			}

			// NbN擾
			bool isDown(Mouse::CLICK click) {return !!(cur.flag & (1 << click));}
			bool isJustDown(Mouse::CLICK click) {return !!(~pre.flag & (1 << click)) && isDown(click);}
			bool isUp(Mouse::CLICK click) {return !!(~cur.flag & (1 << click));}
			bool isJustUp(Mouse::CLICK click) {return !!((pre.flag & (1 << click))) && isUp(click);}
		};


		///////////////////////////
		// t@N^[
		/////
		class Functor {
		public:
			Functor() {}
			virtual ~Functor() {}
			virtual void run() = 0;
		};


		///////////////////////////
		// IuWFNg
		/////
		class Object {
			sp<Region> region;

		protected:
			Param param;
			Vector<sp<Functor> > functors;

		public:
			// }EXANV
			virtual void mouseAction(Mouse &mouse) = 0;

		public:
			Object();
			virtual ~Object();

			// ̈ݒ
			void setRegion(sp<Region> region) {this->region = region;}

			// ̈擾
			sp<Region> getRegion() {return region;}

			// p[^ݒ
			void setParam(const Param &param) {this->param = param;}

			// p[^擾
			const Param &getParam() const {return param;}

			// _̈ɂH
			bool isContain(const Float2 &pos);

			// t@N^[o^
			void registerFunctor(sp<Functor> functor);
		};
	}
}

#endif
